
;**********************************************************************
;                                                                     *
;    Filename:	    AH4Control.asm                                    *
;    Date:                                                            *
;    File Version:                                                    *
;                                                                     *
;    Author: Hirofumi Momose / JJ2PNX                                 *
;    Company:                                                         *
;                                                                     *
;                                                                     *
;**********************************************************************
;                                                                     *
;    Files required:                                                  *
;                                                                     *
;                                                                     *
;                                                                     *
;**********************************************************************
;                                                                     *
;    Notes:                                                           *
;                                                                     *
;                                                                     *
;                                                                     *
;                                                                     *
;**********************************************************************

	list      p=16f819           ; list directive to define processor
	#include <p16F819.inc>        ; processor specific variable definitions

	errorlevel  -302              ; suppress message 302 from list file

	__CONFIG   _CP_OFF & _WRT_ENABLE_OFF & _CPD_OFF & _CCP1_RB2 & _DEBUG_OFF & _LVP_OFF & _BODEN_OFF & _MCLR_OFF & _WDT_OFF & _PWRTE_ON & _INTRC_IO 

; '__CONFIG' directive is used to embed configuration word within .asm file.
; The lables following the directive are located in the respective .inc file.
; See data sheet for additional information on configuration word settings.


; PORTA Bits
ref_in				EQU		H'0000'		; Analog V reflected in
fwd_in				EQU		H'0001'		; Analog V forward in
vref_lo				EQU		H'0002'		; Analog V reference low
vref_hi				EQU		H'0003'		; Analog V reference high
att2				EQU		H'0004'		; Attenuator 2 (Pmax=30W)
ah4_key				EQU		H'0005'		; AH-4 Key signal
ah4_start			EQU		H'0006'		; AH-4 Start signal
att1				EQU		H'0007'		; Attenuator 1 (Pmax=100W)

; PORTB Bits
led_txdet			EQU		H'0000'		; TX DET LED 
led_swr15			EQU		H'0001'		; SWR 1.5 LED
led_swr20			EQU		H'0002'		; SWR 2.0 LED
led_auto			EQU		H'0003'		; AUTO LED
txdet				EQU		H'0004'		; TX DET
sw_auto				EQU		H'0005'		; AUTO SW
sw_tune				EQU		H'0006'		; TUNE SW
led_tune			EQU		H'0007'		; TUNE LED

; File Registers
fr_led				EQU		0x40		; LED debugging/numeric display reg
fr_wait				EQU		0x41		; Use with calling wait_ms
fr_temp				EQU		0x42		; Multiporpose temp reg
fr_vfwd_offset		EQU		0x43		; VFWD offset
fr_vref_offset		EQU		0x44		; VREF offset
fr_vfwd				EQU		0x45		; Measured VFWD
fr_vref				EQU		0x46		; Measured VREF
fr_vswr				EQU		0x47		; Calculated VSWR
fr_vmax_hi			EQU		0x48		; Temp use in calc VSWR
fr_vmax_lo			EQU		0x49		; Temp use in calc VSWR
fr_vmin				EQU		0x4a		; Temp use in calc VSWR
fr_wait_temp		EQU		0x4b		; Temp use in wait_ms
fr_conf_power_thrs	EQU		0x4c		; MIN working power
fr_conf_vswr_thrs	EQU		0x4d		; Tuning start VSWR threshold
fr_vswr_over_count	EQU		0x4e		; VSWR over counter
fr_post_tune		EQU		0x4f		; Post tune flag
fr_blink_interval	EQU		0x50		; LED blinking interval counter
fr_eeadr			EQU		0x51		; EEPROM Address
fr_eedata			EQU		0x52		; EEPROM Data
fr_isr_run_timer	EQU		0x53		; ISR running timer
fr_auto_sw_timer	EQU		0x54

fr_auto_sw_latch	EQU		0x55		; AUTO sw latch
PRV_PRESSED			EQU		0x01		;  Latch state pressed
LATCHED				EQU		0x02		;  Latch state latched

fr_conf_sel			EQU		0x56		; Config current selection
fr_conf_val			EQU		0x57		; Config current value
fr_wait_relay		EQU		0x58		; Relay / TX stable timer

fr_rq				EQU		0x59		; AH4 Control request
RQ_NONE				EQU		H'0000'		; Request: Nothing
RQ_TUNE				EQU		H'0001'		; Request: Tuning
RQ_UNTUNE			EQU		H'0002'		; Request: UN-Tuning

fr_st				EQU		0x5a		; AH4 Control state
ST_UNTUNED			EQU		H'0000'		; State: Untuned
ST_TUNED			EQU		H'0001'		; State: Tuned
ST_TUNE_REQ			EQU		H'0002'		; State: Waiting for AH-4 tuning response
ST_UNTUNE_REQ		EQU		H'0003'		; State: Now in untune operation
ST_TUNING			EQU		H'0004'		; State: Now in tuning operation
ST_CHECK			EQU		H'0005'		; State: Now in result checking

fr_ah4_timer		EQU		0x5b		; AH4 Control timer

; Power constant (offset cancelled)
PVALUE_15W			EQU		D'80'
PVALUE_20W			EQU		D'93'
PVALUE_45W			EQU		D'139'
PVALUE_55W			EQU		D'154'

; EEPROM Address
EEADR_WAIT_RELAY	EQU		0x00
EEADR_VSWR_THRS		EQU		0x01
EEADR_POWER_THRS	EQU		0x02
EEADR_AUTO			EQU		0x03

; EEPROM Default data
EEDATA_WAIT_RELAY	EQU		0x00		; WAIT = 100 ms
EEDATA_VSWR_THRS	EQU		0x03		; VSWR = 2.0
EEDATA_POWER_THRS	EQU		0x05		; POWR = 5 watts
EEDATA_AUTO			EQU		0x01		; MODE = AUTOMATIC

; ---------------------------------------------------------
; Vector
; Input  : None
; Output : None
; Temp   : None
; Remks  : 
; ---------------------------------------------------------
	ORG     0x000				; Reset vector
	goto    main				; go to main

	ORG     0x004				; Interrupt vector
	goto	isr_main			; go to interrupt routine


; ---------------------------------------------------------
; main
; Input  : None
; Output : None
; Temp   : None
; Remks  : 
; ---------------------------------------------------------
main
	bsf		STATUS, RP0			; Select Bank1
	movlw	B'01100000'			; Setup INT OSC at 4MHz (See DS p37)
	movwf	OSCCON
	bcf		OPTION_REG, NOT_RBPU
	
	movlw	B'00101111'			; Setup PORTA ATT1/AH4Start/AH4Key/ATT2/VR+/VR-/FWD/REF 
	movwf	TRISA

	movlw	B'01110000'			; Setup PORTB led_tune/TUNE/AUTO/TX/FAT/S20/S15/S12
	movwf	TRISB

	movlw	B'00001101'			; Configure A/D inputs (See DS p82)
	movwf	ADCON1

	; Clear ALL I/O port
	banksel	PORTA
	clrf	PORTA
	clrf	PORTB

; Measure offset voltage
; ---------------------------------------------------------
measure_offset
	; If TX waiting RX
	btfsc	PORTB, txdet
	goto	measure_offset

	; Wait 100 ms for analog stability
	movlw	D'100'				
	movwf	fr_wait
	call	wait_ms
	; Analog measurement
	call	measure
	; Store offset
	movfw	fr_vfwd
	movwf	fr_vfwd_offset;
	movfw	fr_vref
	movwf	fr_vref_offset;

; Check system configuration mode
; ---------------------------------------------------------
	; Press AUTO during POR -> Configuration mode.
	btfss	PORTB, sw_auto
	goto	main_config

; Load config data from EEPROM
; ---------------------------------------------------------
	; Load wait_relay
	movlw	EEADR_WAIT_RELAY
	movwf	fr_eeadr
	call	eeprom_read
	movwf	fr_wait_relay

	; Load VSWR threshold
	movlw	EEADR_VSWR_THRS
	movwf	fr_eeadr
	call	eeprom_read
	call	get_vswr_table
	movwf	fr_conf_vswr_thrs

	; Load minimum working power
	movlw	EEADR_POWER_THRS
	movwf	fr_eeadr
	call	eeprom_read
	call	get_power_thrs_table
	movwf	fr_conf_power_thrs

	; Load AUTO mode
	movlw	EEADR_AUTO
	movwf	fr_eeadr
	call	eeprom_read
	btfss	STATUS, Z
	bsf		PORTB, led_auto

; Initialize state
; ---------------------------------------------------------
	movlw	ST_UNTUNED
	movwf	fr_st
	movlw	RQ_NONE
	movwf	fr_rq
	clrf	fr_ah4_timer

; Enable interrupt and halt cpu
; ---------------------------------------------------------
	banksel	INTCON
	bsf		INTCON, RBIE
	bsf		INTCON, GIE
	sleep


; Configuration mode
; ---------------------------------------------------------
main_config

	btfss	PORTB, sw_tune
	goto	main_config_all_default

main_config_0
	; Initialize
	clrf	fr_conf_sel
	clrf	fr_conf_val
	clrf	fr_temp
	bsf		fr_temp, led_auto
	movfw	fr_temp
	movwf	PORTB

main_config_1
	; Key dispatcher
	btfss	PORTB, sw_auto
	goto	main_config_2
	btfss	PORTB, sw_tune
	goto	main_config_4
	goto	main_config_1

main_config_2
	; AUTO switch pressed
	; SW Stability
	movlw	D'20'
	movwf	fr_wait
	call	wait_ms
	btfss	PORTB, sw_auto
	goto	main_config_1

	; AUTO SW latch
	btfsc	PORTB, led_auto
	goto	main_config_3

	; AUTO LED OFF->ON
	movfw	PORTB
	movwf	fr_temp
	bsf		fr_temp, led_auto
	movfw	fr_temp
	movwf	PORTB

	; Load current data from EEPROM
	movfw	fr_conf_sel
	movwf	fr_eeadr
	call	eeprom_read
	movwf	fr_conf_val

	; Display current data
	movfw	fr_conf_val
	movwf	fr_led
	call	displed_num

	goto	main_config_1

main_config_3
	; AUTO LED ON->OFF
	movfw	PORTB
	movwf	fr_temp
	bcf		fr_temp, led_auto
	movfw	fr_temp
	movwf	PORTB

	; Display current selection
	movfw	fr_conf_sel
	movwf	fr_led
	call	displed_num

	goto	main_config_1

main_config_4
	; TUNE SW pressed
	; SW Stability
	movlw	D'20'
	movwf	fr_wait
	call	wait_ms
	btfss	PORTB, sw_tune
	goto	main_config_1

	btfsc	PORTB, led_auto
	goto	main_config_5

	; AUTO LED OFF (Select)
	incf	fr_conf_sel, F
	movfw	fr_conf_sel
	sublw	D'3'
	btfsc	STATUS, Z
	clrf	fr_conf_sel

	; Display current selection
	movfw	fr_conf_sel
	movwf	fr_led
	call	displed_num

	goto	main_config_1

main_config_5
	; AUTO LED ON(Change value)

	; Inc Value
	incf	fr_conf_val, F
	movfw	fr_conf_val
	sublw	D'8'
	btfsc	STATUS, Z
	clrf	fr_conf_val
	btfss	STATUS, C
	clrf	fr_conf_val

	; Display current data
	movfw	fr_conf_val
	movwf	fr_led
	call	displed_num

	; Save data to EEPROM
	movfw	fr_conf_sel
	movwf	fr_eeadr
	movfw	fr_conf_val
	movwf	fr_eedata
	call	eeprom_write

	goto	main_config_1

main_config_all_default
	; Setup all default
	movlw	EEADR_WAIT_RELAY
	movwf	fr_eeadr
	movlw	EEDATA_WAIT_RELAY
	movwf	fr_eedata
	call	eeprom_write

	movlw	EEADR_VSWR_THRS
	movwf	fr_eeadr
	movlw	EEDATA_VSWR_THRS
	movwf	fr_eedata
	call	eeprom_write

	movlw	EEADR_POWER_THRS
	movwf	fr_eeadr
	movlw	EEDATA_POWER_THRS
	movwf	fr_eedata
	call	eeprom_write

	movlw	EEADR_AUTO
	movwf	fr_eeadr
	movlw	EEADR_AUTO
	movwf	fr_eedata
	call	eeprom_write

	clrf	fr_temp
	bsf		fr_temp, led_auto
	bsf		fr_temp, led_tune
	movfw	fr_temp
	movwf	PORTB

main_config_all_default_0
	goto	main_config_all_default_0

; End of MAIN program
; ---------------------------------------------------------


; ---------------------------------------------------------
; Interrupt handler
; Input  : None
; Output : None
; Temp   : None
; Remks  : Invoked when
;          1) Press or release any key.
;          2) TX detection
; ---------------------------------------------------------
isr_main
	; Do initialize
	banksel	0x00
	; Clear post_tune
	clrf	fr_post_tune
	; Clear request status
	clrf	fr_rq
	; Clear VSWR threshold over count
	clrf	fr_vswr_over_count
	; Clear ah4_start
	bcf		PORTA, ah4_start
	; clean fr_st
	movfw	fr_st			; if(status != ST_TUNED)
	sublw	ST_TUNED			;  then status <= ST_UNTUNED
	btfss	STATUS, Z
	clrf	fr_st
	; Clear AUTO switch state
	clrf	fr_auto_sw_latch
	; Setup ISR minimun running timer
	movlw	D'100'
	movwf	fr_isr_run_timer
	; Wait for stability
	movlw	D'10'
	movwf	fr_wait
	call	wait_ms

; Main interrupt service routine loop head
; ---------------------------------------------------------
isr_main_loop_head

; Handle FULLAUTO / SEMIAUTO switch
; ---------------------------------------------------------
	btfss	PORTB, sw_auto
	goto	isr_main_fs_sw_now_pressed
	
isr_main_fs_sw_now_released
	; Now released
	btfsc	fr_auto_sw_latch, PRV_PRESSED
	goto	isr_main_fs_sw_prev_pressed
	decfsz	fr_auto_sw_timer, F
	goto	isr_main_loop_0
	; SW 10ms countinuosly released
	; Procedure start
	bcf		fr_auto_sw_latch, LATCHED
	; Procedure end
	goto	isr_main_loop_0
	
isr_main_fs_sw_prev_pressed
	bcf		fr_auto_sw_latch, PRV_PRESSED
	movlw	D'10'
	movwf	fr_auto_sw_timer
	goto	isr_main_loop_0

isr_main_fs_sw_now_pressed
	; Now pressed
	btfss	fr_auto_sw_latch, PRV_PRESSED
	goto	isr_main_fs_sw_prev_released
	decfsz	fr_auto_sw_timer, F
	goto	isr_main_loop_0
	; SW 10ms continuously pressed
	; Procedure start
	; One shot latch
	btfsc	fr_auto_sw_latch, LATCHED

	goto	isr_main_loop_0
	bsf		fr_auto_sw_latch, LATCHED	
	; One shot latch end

	movlw	EEADR_AUTO
	movwf	fr_eeadr
	btfsc	PORTB, led_auto
	goto	isr_main_fs_semiauto
isr_main_fs_fullauto
	bsf		PORTB, led_auto
	movlw	0x01
	movwf	fr_eedata
	call	eeprom_write
	goto	isr_main_loop_0
isr_main_fs_semiauto
	bcf		PORTB, led_auto
	clrf	fr_eedata
	call	eeprom_write
	goto	isr_main_loop_0
	; Procedure end

isr_main_fs_sw_prev_released
	bsf		fr_auto_sw_latch, PRV_PRESSED
	movlw	D'10'
	movwf	fr_auto_sw_timer
	goto	isr_main_loop_0

; Dispatcher
; ---------------------------------------------------------
isr_main_loop_0

	; Analog measurement and get VSWR	
	call	measure
	call	remove_offset
	call	calc_vswr

	; Check tuning in progress
	movfw	fr_rq
	sublw	RQ_TUNE
	btfsc	STATUS, Z
	goto	isr_main_now_tuning

	; Check something in progress.
	movf	fr_rq, F
	btfss	STATUS, Z
	goto	isr_main_common

	; Check post_tune
	movf	fr_post_tune, F
	btfss	STATUS, Z
	goto	isr_main_post_tune

	; Check Transmission
	btfsc	PORTB, txdet
	goto	isr_main_txdet

	; Check TUNE switch (under RX)
	btfss	PORTB, sw_tune
	goto	isr_main_untune			; Start untune

	; ISR minnimum free running timer
	decfsz	fr_isr_run_timer, F		; if(fr_isr_run_timer -- == 0) goto isr_main_sleep
	goto	isr_main_common
	goto	isr_main_sleep


; Transmit detection
; ---------------------------------------------------------
isr_main_txdet

	; Check TUNE switch (under transmission)
	btfss	PORTB, sw_tune
	goto	isr_main_start_tune		; Start tune manually

	; Start automatic tuning judgement
	; Check whether full automatic mode.
	btfss	PORTB, led_auto			; if(not led_auto) goto isr_main_common 
	goto	isr_main_common

	; Check minumum power requirement
	movfw	fr_vfwd					; if(fr_vfwd <= fr_conf_power_thrs)
	subwf	fr_conf_power_thrs, W		;  goto isr_main_common
	btfsc	STATUS, C				; Clear over count
	clrf	fr_vswr_over_count		;  when detecting under-power
	btfsc	STATUS, C
	goto	isr_main_common

	; Check whether CURRENT VSWR > THRESHOLD continuously.
	movfw	fr_conf_vswr_thrs			; if(fr_conf_vswr_thrs <= fr_vswr)
	subwf	fr_vswr, W				;   fr_vswr_over_count ++ 
	btfsc	STATUS, C				; else
	incf	fr_vswr_over_count,F	;   fr_vswr_over_count =0
	btfss	STATUS, C
	clrf	fr_vswr_over_count
	movlw	D'10'					; if(fr_vswr_over_count != 10)
	subwf	fr_vswr_over_count, W	;   goto isr_main_common
	btfss	STATUS, Z				; else
	goto	isr_main_common			;   fr_rq = RQ_TUNE


; Start ATU Tuning
; ---------------------------------------------------------
isr_main_start_tune

	; Enable att1 and att2
	movfw	PORTA
	movwf	fr_temp
	bsf		fr_temp, att1
	bsf		fr_temp, att2
	movfw	fr_temp
	movwf	PORTA
	
	; Call relay stable wait 
	call	wait_relay
	
	; Re-Measure for getting accurate TX power
	call	measure
	call	remove_offset
	
	; Disable att according to TX power
	movfw	PORTA
	movwf	fr_temp
	movfw	fr_vfwd					; if(fr_vfwd <= PVALUE_15W)
	sublw	PVALUE_15W				;  then off att1
	btfsc	STATUS, C
	bcf		fr_temp, att1
	movfw	fr_vfwd					; if(fr_vfwd <= PVALUE_45W)
	sublw	PVALUE_45W				;  then off att2
	btfsc	STATUS, C
	bcf		fr_temp, att2
	movfw	fr_temp
	movwf	PORTA

	; Call relay stable wait 
	call	wait_relay

	; Request TUNE
	movlw	RQ_TUNE
	movwf	fr_rq
	
	; Set post_tune
	incf	fr_post_tune, F

	; Goto common routine
	goto	isr_main_common


; Post tune (After tuning process)
; ---------------------------------------------------------
isr_main_post_tune
	
	; Clear post_tune
	clrf	fr_post_tune

	; Disable att1 and att2
	movfw	PORTA
	movwf	fr_temp
	bcf		fr_temp, att1
	bcf		fr_temp, att2
	movfw	fr_temp
	movwf	PORTA

	; Check tuning result
	movfw	fr_st				; if tunig failed
	sublw	ST_UNTUNED			;  goto isr_main_post_tune_0
	btfsc	STATUS, Z
	goto	isr_main_post_tune_0

	; Waiting for relay stability
	call	wait_relay

	; Disable full automatic mode, if tuned with vswr over threashold
	; Measurement and get VSWR
	call	measure
	call	remove_offset
	call	calc_vswr

	; Compare vswr with threshold
	movfw	fr_conf_vswr_thrs
	subwf	fr_vswr, W
	btfss	STATUS ,C
	goto	isr_main_common		; OK under threshold

isr_main_post_tune_0

	btfss	PORTB, txdet
	goto	isr_main_common

	; Disable full automatic mode
	; This is temporarily. No EEPROM update
	movfw	PORTB
	movwf	fr_temp
	bcf		fr_temp, led_auto
	movfw	fr_temp
	movwf	PORTB

	; Goto common routine
	goto	isr_main_common


; ATU UN-Tuning
; ---------------------------------------------------------
isr_main_untune

	; Request UNTUNE
	movlw	RQ_UNTUNE				; fr_st <= RQ_UNTUNE
	movwf	fr_rq
	; Goto common routine
	goto	isr_main_common


; ATU Tuning in progress
; ---------------------------------------------------------
isr_main_now_tuning

	; Check current ATT settings.
	btfsc	PORTA, att1
	goto	isr_main_now_tuning_0

	; NO ATT enabled (Power range must be 5 to 15 watts)
	movfw	fr_vfwd
	sublw	PVALUE_20W
	btfsc	STATUS, C
	goto	isr_main_common	; OK under 20 watts.

	; Over Power detected. Enable ATT for ATU protection.
	movfw	PORTA
	movwf	fr_temp
	bsf		fr_temp, att1
	bsf		fr_temp, att2
	movfw	fr_temp
	movwf	PORTA
	goto	isr_main_common

isr_main_now_tuning_0
	; Check ATT2
	btfsc	PORTA, att2
	goto	isr_main_common
	; Only ATT1 enabled (Power range must be 15 to 45 watts)
	movfw	fr_vfwd
	sublw	PVALUE_55W
	btfsc	STATUS, C
	goto	isr_main_common	; OK under 55 watts.

	; Over Power detected. Enable ATT for ATU protection
	movfw	PORTA
	movwf	fr_temp
	bsf		fr_temp, att1
	bsf		fr_temp, att2
	movfw	fr_temp
	movwf	PORTA
	goto	isr_main_common


; Interrupt service loop common routine
; ---------------------------------------------------------
isr_main_common

	; Update tuner control status
	call	control_tuner

	; Display TUNED status LED
	call	displed_tune

	; Display SWR monitoring LED
	call	displed_swr

	; LED blinking scaling counter (99 -> 0)
	movf	fr_blink_interval, F
	btfss	STATUS, Z
	goto	isr_main_blink_nonzero
	movlw	D'100'
	movwf	fr_blink_interval
isr_main_blink_nonzero
	decf	fr_blink_interval, F

	; 1ms wait timer (Main loop tick timer)
	movlw	0x01
	movwf	fr_wait
	call	wait_ms
	
	; Go to loop head
	goto	isr_main_loop_head


; Interrupt service finalizer
; ---------------------------------------------------------
isr_main_sleep

	; Cleanup PORTA
	movfw	PORTA
	movwf	fr_temp
	bcf		fr_temp, ah4_start
	bcf		fr_temp, att1
	bcf		fr_temp, att2
	movfw	fr_temp
	movwf	PORTA

	; Cleanup PORTB
	movfw	PORTB
	movwf	fr_temp
	bcf		fr_temp, led_txdet
	bcf		fr_temp, led_swr15
	bcf		fr_temp, led_swr20
	movfw	fr_temp
	movwf	PORTB

	; Enable Interrupt
	bcf		INTCON, RBIF
	bsf		INTCON, GIE
	
	; CPU Halt
	sleep
	; No more instruction left.


; ---------------------------------------------------------
; Relay / TX power stable wait timer
; Input  : fr_wait_relay
; Output : None
; Temp   : fr_temp
; Remks  : Wait time = 100 ms *(1 + fr_wait_relay) 
; ---------------------------------------------------------
wait_relay
	movfw	fr_wait_relay
	movwf	fr_temp
	incf	fr_temp, F
	incf	fr_temp, F
wait_relay_0
	decfsz	fr_temp, F
	goto	wait_relay_1
	return
wait_relay_1
	movlw	D'100'
	movwf	fr_wait
	call	wait_ms
	goto	wait_relay_0


; ---------------------------------------------------------
; Display TUNED LED
; Input  : fr_st
; Output : PORTB.led_tune
; Temp   : fr_temp, fr_displed_interval
; Remks  :
; ---------------------------------------------------------
displed_tune

	movfw	PORTB
	movwf	fr_temp

	movfw	fr_st					; if(fr_st == ST_TUNED) turn on
	sublw	ST_TUNED
	btfsc	STATUS, Z
	goto	displed_tune_turn_on

	movfw	fr_st					; if(fr_st == ST_UNTUNED) turn off
	sublw	ST_UNTUNED
	btfsc	STATUS, Z
	goto	displed_tune_turn_off

	movfw	fr_st					; if(fr_st == ST_UNTUNE_REQ) turn off
	sublw	ST_UNTUNE_REQ
	btfsc	STATUS, Z
	goto	displed_tune_turn_off

	; Others blinking LED
	movf	fr_blink_interval, F
	btfss	STATUS, Z
	return

	btfss	fr_temp, led_tune
	goto	displed_tune_turn_on

displed_tune_turn_off
	bcf		fr_temp, led_tune
	movfw	fr_temp
	movwf	PORTB
	return

displed_tune_turn_on	
	bsf		fr_temp, led_tune
	movfw	fr_temp
	movwf	PORTB
	return


; ---------------------------------------------------------
; Display VSWR LED
; Input  : fr_vfwd, fr_vswf, fr_conf_power_thrs,
;        : fr_blink_interval
; Output : PORTB.led_txdet PORTB.led_swr15 PORTB.led_swr20
; Temp   : fr_temp
; Remks  :
; ---------------------------------------------------------
displed_swr
	; Works only under transmission
	btfss	PORTB, txdet
	return

	movfw	PORTB
	movwf	fr_temp

	; TX Power check
	movfw	fr_vfwd
	subwf	fr_conf_power_thrs, W
	btfss	STATUS, C
	goto	displed_txdet_turn_on		; go turn on when enough power

	; Blinking swr12 led (below required power)
	movf	fr_blink_interval, F
	btfss	STATUS, Z
	goto	displed_swr1520
	
	btfss	fr_temp, led_txdet
	goto	displed_txdet_turn_on
	
	bcf		fr_temp, led_txdet
	goto	displed_swr1520

displed_txdet_turn_on
	bsf		fr_temp, led_txdet			; Lit green (Tx monitor)

displed_swr1520
	movfw	fr_vswr						; if(fr_vswr >= 15) turn on yellow
	sublw	D'14'
	btfss	STATUS, C
	bsf		fr_temp, led_swr15
	btfsc	STATUS, C
	bcf		fr_temp, led_swr15

	movfw	fr_vswr						; if(fr_vswr >= 20) turn on red
	sublw	D'19'
	btfss	STATUS, C
	bsf		fr_temp, led_swr20
	btfsc	STATUS, C
	bcf		fr_temp, led_swr20

	movfw	fr_temp
	movwf	PORTB
	return


; ---------------------------------------------------------
; Display register value by LED (For debug / config)
; Input  : fr_led
; Output : PORTB.led(n)
; Temp   : fr_temp
; Remks  :
; ---------------------------------------------------------
displed_num
	movfw	PORTB
	movwf	fr_temp

	; For Debugging
	; goto	displed_num_debug

	btfsc	fr_led, 2
	bsf		fr_temp, led_txdet
	btfss	fr_led, 2
	bcf		fr_temp, led_txdet

	btfsc	fr_led, 1
	bsf		fr_temp, led_swr15
	btfss	fr_led, 1
	bcf		fr_temp, led_swr15

	btfsc	fr_led, 0
	bsf		fr_temp, led_swr20
	btfss	fr_led, 0
	bcf		fr_temp, led_swr20

	movfw	fr_temp
	movwf	PORTB
	return

	; Following is debug use
displed_num_debug

	btfss	PORTB, sw_tune		; Push to display lower 4bits
	goto	displed_num_debug_lo
	
	btfsc	fr_led, 7
	bsf		fr_temp, led_txdet
	btfss	fr_led, 7
	bcf		fr_temp, led_txdet

	btfsc	fr_led, 6
	bsf		fr_temp, led_swr15
	btfss	fr_led, 6
	bcf		fr_temp, led_swr15

	btfsc	fr_led, 5
	bsf		fr_temp, led_swr20
	btfss	fr_led, 5
	bcf		fr_temp, led_swr20

	btfsc	fr_led, 4
	bsf		fr_temp, led_auto
	btfss	fr_led, 4
	bcf		fr_temp, led_auto

	movfw	fr_temp
	movwf	PORTB
	return

displed_num_debug_lo
	btfsc	fr_led, 3
	bsf		fr_temp, led_txdet
	btfss	fr_led, 3
	bcf		fr_temp, led_txdet

	btfsc	fr_led, 2
	bsf		fr_temp, led_swr15
	btfss	fr_led, 2
	bcf		fr_temp, led_swr15

	btfsc	fr_led, 1
	bsf		fr_temp, led_swr20
	btfss	fr_led, 1
	bcf		fr_temp, led_swr20

	btfsc	fr_led, 0
	bsf		fr_temp, led_auto
	btfss	fr_led, 0
	bcf		fr_temp, led_auto

	movfw	fr_temp
	movwf	PORTB
	return


; ---------------------------------------------------------
; Measure FWD/REF voltage
; Input  : None
; Output : fr_vfwd, fr_vref
; Temp   : No temp used
; Remks  : 8 bit resolution
; ---------------------------------------------------------
measure
	; Measure Reflection voltage
	movlw	B'11000001'			; RC Clock, Enable A/D, Select channel 0
	movwf	ADCON0
	call	wait_10us			; wait 20us for sampling (Approx 15us required)
	call	wait_10us
	bsf		ADCON0, GO			; Start A/D Conversion

measure_0						; Waiting for A/D completion
	btfsc	ADCON0, GO			
	goto	measure_0

	movfw	ADRESH				; Copy result to fr_vref
	movwf	fr_vref

	; Measure Forward voltage
	movlw	B'11001001'			; RC Clock, Enable A/D, select channel 1
	movwf	ADCON0
	call	wait_10us			; Wait 20us for sampling (Approx 15us required)
	call	wait_10us
	bsf		ADCON0, GO			; Start A/D conversion

measure_1						; Waiting for A/D completion
	btfsc	ADCON0, GO
	goto	measure_1

	movfw	ADRESH				; Copy result to fr_vfwd
	movwf	fr_vfwd

	bcf		ADCON0, ADON		; Disable A/D	
	return


; ---------------------------------------------------------
; Calculate VSWR
; Input  : fr_vfwd, fr_vref
; Output : fr_vswr
; Temp   : fr_vmax_hi, fr_vmax_lo, fr_vmin  
; Remks  : VSWR = 10 * (fr_vfwd + fr_vref)/(fr_vfwd - fr_vref)
; ---------------------------------------------------------
calc_vswr
	movlw	0xff
	movwf	fr_vswr

	; Vmin = 0 (Div by zero... return MAX
	; if(vref - vfwd >=0) return MAX
	movfw	fr_vfwd
	subwf	fr_vref, W
	btfsc	STATUS, C
	return

	; Vmax(hi::lo) = (vfwd + vref) * 10
	clrf	fr_vmax_hi
	movfw	fr_vfwd
	movwf	fr_vmax_lo
	movfw	fr_vref
	addwf	fr_vmax_lo, F
	btfsc	STATUS, C
	incf	fr_vmax_hi, F
	; *2
	bcf		STATUS, C
	rlf		fr_vmax_hi, F
	rlf		fr_vmax_lo, F
	btfsc	STATUS, C
	incf	fr_vmax_hi, F
	; *2
	bcf		STATUS, C
	rlf		fr_vmax_hi, F
	rlf		fr_vmax_lo, F
	btfsc	STATUS, C
	incf	fr_vmax_hi, F
	; *2
	bcf		STATUS, C
	rlf		fr_vmax_hi, F
	rlf		fr_vmax_lo, F
	btfsc	STATUS, C
	incf	fr_vmax_hi, F
	; +vfwd
	movfw	fr_vfwd
	addwf	fr_vmax_lo, F
	btfsc	STATUS, C
	incf	fr_vmax_hi, F
	; +vfwd
	movfw	fr_vfwd
	addwf	fr_vmax_lo, F
	btfsc	STATUS, C
	incf	fr_vmax_hi, F
	; +vref
	movfw	fr_vref
	addwf	fr_vmax_lo, F
	btfsc	STATUS, C
	incf	fr_vmax_hi, F
	; +vref
	movfw	fr_vref
	addwf	fr_vmax_lo, F
	btfsc	STATUS, C
	incf	fr_vmax_hi, F
	
	; Vmin = vfwd - vref
	movfw	fr_vfwd
	movwf	fr_vmin
	movfw	fr_vref
	subwf	fr_vmin, F
	
	clrf	fr_vswr

	; Do primitive division
calc_vswr_loop
	; Result limitter (Save CPU cycles)
	; No more result (over 30) required
	movfw	fr_vswr
	sublw	D'30'
	btfsc	STATUS, Z
	return

	movfw	fr_vmin
	subwf	fr_vmax_lo, F
	btfsc	STATUS, C
	incf	fr_vswr, F
	btfsc	STATUS, C
	goto	calc_vswr_loop
	
	movf	fr_vmax_hi, F
	btfsc	STATUS, Z
	return
	decf	fr_vmax_hi, F
	incf	fr_vswr, F
	goto	calc_vswr_loop


; ---------------------------------------------------------
; Remove VFWD, VREF offset voltage
; Input  : fr_vfwd, fr_vref, fr_vfwd_offset, fr_vref_offset
; Output : fr_vfwd, fr_vref
; Temp   : None
; Remks  :
; ---------------------------------------------------------
remove_offset
	; Remove offset in fr_vfwd
	movfw	fr_vfwd_offset			; if(fr_vfwd > fr_vfwd_offset)
	subwf	fr_vfwd, F				;  fr_vfwd = fr_vfwd - fr_vfwd_offset; 
	btfss	STATUS, C				; else
	clrf	fr_vfwd					;  fr_vfwd = 0;

	; Remove offset in fr_vref 
	movfw	fr_vref_offset			; if(fr_vref > fr_vref_offset)
	subwf	fr_vref, F				;  fr_vref = fr_vref - fr_vref_offset;
	btfss	STATUS, C				; else
	clrf	fr_vref					;  fr_vref = 0;
	return


; ---------------------------------------------------------
; Wait for ms
; Input  : fr_wait
; Output : None
; Temp   : fr_wait_temp
; Remks  : Wait for fr_wait ms
; ---------------------------------------------------------
wait_ms
wait_ms_100us
	movlw	0xf7
	movwf	fr_wait_temp
wait_ms_100us_1
	nop
	decfsz	fr_wait_temp, f
	goto	wait_ms_100us_1
	nop
	decfsz	fr_wait, f
	goto	wait_ms
	return


; ---------------------------------------------------------
; Wait for 10us
; Input  : None
; Output : None
; Temp   : None  
; Remks  : 
; ---------------------------------------------------------
wait_10us
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	return


; ---------------------------------------------------------
; Function     : Control AH-4 Tuner
; Input reg    : fr_rq
; Output reg   : fr_rq, fr_st
; Internal reg : fr_ah4_timer
; Remarks      : 
;              ; Control register fr_rq
; Remakrs      :    RQ_TUNE       : To start tuning. 
;              :    RQ_UNTUNE     : To start untune. 
;              :    Cleared automatically when operation is done.
;              :
;              : Status register fr_st (Do not change)
;              :    ST_UNTUNED    : Now in untuned condition.
;              :    ST_TUNED      : Now in tuned condition.
;              :    ST_TUNE_REQ   : Now requesting to be tuned.
;              :    ST_UNTUNE_REQ : Now requesting to be untuned.
;              :    ST_TUNING     : Tuning now.
;              :    ST_CHECK      : Checking tuner result
; POR Init     : fr_rq <= RQ_NONE (0)
;              : fr_st <= ST_UNTUNED (0)
;              : fr_ah4_timer <= 0
; ---------------------------------------------------------
control_tuner
	; Dispatcher
	; ST_UNTUNED
	movfw	fr_st
	sublw	ST_UNTUNED
	btfsc	STATUS, Z
	goto	control_tuner_st_untuned
	
	; ST_TUNED
	movfw	fr_st
	sublw	ST_TUNED
	btfsc	STATUS, Z
	goto	control_tuner_st_tuned
	
	; ST_TUNE_REQ
	movfw	fr_st
	sublw	ST_TUNE_REQ
	btfsc	STATUS, Z
	goto	control_tuner_st_tune_req

	; ST_UNTUNE_REQ
	movfw	fr_st
	sublw	ST_UNTUNE_REQ
	btfsc	STATUS, Z
	goto	control_tuner_st_untune_req
	
	; ST_TUNING
	movfw	fr_st
	sublw	ST_TUNING
	btfsc	STATUS, Z
	goto	control_tuner_st_tuning

	; OTHERS
	goto	control_tuner_st_check
	
control_tuner_st_tuned
control_tuner_st_untuned
	decfsz	fr_ah4_timer, F					; if(--timer > 0) return
	return
	movfw	fr_rq						; if(fr_rq == RQ_UNTUNE)
	sublw	RQ_UNTUNE						;  goto control_tuner_st_untune_req_1
	btfsc	STATUS, Z
	goto	control_tuner_st_untune_req_1

	movfw	fr_rq						; if(fr_rq != RQ_TUNE)
	sublw	RQ_TUNE							;  return
	btfss	STATUS, Z
	return
											; In case of (fr_rq == RQ_TUNE)
	bsf		PORTA, ah4_start				; AH-4_start <= 1
	movlw	ST_TUNE_REQ						; state <= ST_TUNE_REQ					
	movwf	fr_st
	goto	control_tuner_st_tune_req

control_tuner_st_untune_req_1
	movlw	ST_UNTUNE_REQ					; state <= ST_UNTUNE_REQ
	movwf	fr_st
	movlw	D'50'							; timer <= 50 ms
	movwf	fr_ah4_timer
	bsf		PORTA, ah4_start				; AH4_start <= 1
	goto	control_tuner_st_untune_req

control_tuner_st_tune_req
	btfsc	PORTA, ah4_key					; if(ah4_key == 1) return
	return
	bcf		PORTA, ah4_start				; AH4_start <= 0
	movlw	ST_TUNING						; state <= ST_TUNING
	movwf	fr_st

control_tuner_st_tuning
	btfss	PORTA, ah4_key					; if(ah4_key == 0) return
	return
	movlw	ST_CHECK						; state <= ST_CHECK
	movwf	fr_st
	movlw	D'50'							; timer <= 50 ms
	movwf	fr_ah4_timer

control_tuner_st_check
	decfsz	fr_ah4_timer, F					; if(--timer > 0) return
	return
	btfss	PORTA, ah4_key					; if(ah4_key == 0)
	goto	control_tuner_st_check_failed	;  goto control_tuner_st_check_failed
	movlw	ST_TUNED						; state <= ST_TUNED
	movwf	fr_st
	clrf	fr_rq						; Clear request
	movlw	D'200'							; Interval time required to next operation
	movwf	fr_ah4_timer
	return

control_tuner_st_check_failed
	movlw	ST_UNTUNED						; state <= ST_UNTUNED
	movwf	fr_st
	clrf	fr_rq						; Clear request
	movlw	D'200'							; Interval time required to next operation
	movwf	fr_ah4_timer
	return
	
control_tuner_st_untune_req
	decfsz	fr_ah4_timer, F					; if(--timer > 0) return
	return
	movlw	ST_UNTUNED						; state <= ST_UNTUNED
	movwf	fr_st
	bcf		PORTA, ah4_start				; ah4_start <= 0
	clrf	fr_rq						; Clear request		
	return


; ---------------------------------------------------------
; Read data from EEPROM 
; Input  : fr_eeadr (EEPROM ADDRESS)
; Output : w (DATA)
; Temp   : None
; Remks  : Set bank 0 when return
; ---------------------------------------------------------
eeprom_read
	movfw	fr_eeadr
	banksel	EEADR
	movwf	EEADR
	banksel	EECON1
	bcf		EECON1, EEPGD
	bsf		EECON1, RD
	banksel	EEDATA
	movfw	EEDATA
	banksel	0x00
	return


; ---------------------------------------------------------
; Write data into EEPROM
; Input  : fr_eeadr, fr_eedata
; Output : None
; Temp   : None
; Remks  : Set bank 0 when return
; ---------------------------------------------------------
eeprom_write
	banksel	EECON1
eeprom_write_0
	btfsc	EECON1, WR
	goto	eeprom_write_0
	banksel fr_eeadr
	movfw	fr_eeadr
	banksel	EEADR
	movwf	EEADR
	banksel fr_eedata
	movfw	fr_eedata
	banksel	EEDATA
	movwf	EEDATA
	banksel	EECON1
	bcf		EECON1, EEPGD
	bsf		EECON1, WREN
	movlw	0x55
	movwf	EECON2
	movlw	0xaa
	movwf	EECON2
	bsf		EECON1, WR
	bcf		EECON1, WREN
	banksel 0x00
	return

; ---------------------------------------------------------
; Using COMPUTED GOTO (Instruction using PCL) should be 
; care about 8bit memory boundary. 
; Using high address with lower boundary to avoid problems.
	ORG	0x700
; ---------------------------------------------------------

; ---------------------------------------------------------
; SWR configuration table
; Input  : w
; Output : w
; Temp   : fr_temp
; Remks  :
; ---------------------------------------------------------
get_vswr_table
	movwf	fr_temp
	movlw	0x07
	movwf	PCLATH
	movfw	fr_temp
	andlw	0x07
	addwf	PCL, F
	retlw	D'14'	; 0 (SWR 1.4)
	retlw	D'16'	; 1 (SWR 1.6)
	retlw	D'18'	; 2 (SWR 1.8)
	retlw	D'20'	; 3 (SWR 2.0)
	retlw	D'22'	; 4 (SWR 2.2)
	retlw	D'24'	; 5 (SWR 2.4)
	retlw	D'26'	; 6 (SWR 2.6)
	retlw	D'28'	; 7 (SWR 2.8)


; ---------------------------------------------------------
; Minumum Power configuration table
; Input  : w
; Output : w
; Temp   : fr_temp
; Remks  :
; ---------------------------------------------------------
get_power_thrs_table
	movwf	fr_temp
	movlw	0x07
	movwf	PCLATH
	movfw	fr_temp
	andlw	0x07
	addwf	PCL, F
	retlw	D'15'	; 0 (0.5 W)
	retlw	D'21'	; 1 (1 W)
	retlw	D'30'	; 2 (2 W)
	retlw	D'36'	; 3 (3 W)
	retlw	D'42'	; 4 (4 W)
	retlw	D'47'	; 5 (5 W)
	retlw	D'51'	; 6 (6 W)
	retlw	D'56'	; 7 (7 W)


; ---------------------------------------------------------
; EEPROM Default data
; ---------------------------------------------------------
	ORG	0x2100
	DE	EEDATA_WAIT_RELAY		; Wwait
	DE	EEDATA_VSWR_THRS		; VSWR
	DE	EEDATA_POWER_THRS		; Power
	DE	EEDATA_AUTO				; Full Automatic

; ---------------------------------------------------------
; Directive 'end of program'
; ---------------------------------------------------------
	END
